from PyQt5.QtCore import QSize, Qt, pyqtSignal
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import *
import minesweeperGame as MSGame
import controller


def main():
    game = MSGame.minesweeperGame(*GUIInit())
    gameGUI = minesweeperGUI(game)
    gameGUI.startGame()


class RightClickableButton(QPushButton):
    rightClicked = pyqtSignal()
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            QPushButton.mousePressEvent(self, event)
        elif event.button() == Qt.RightButton:
            self.rightClicked.emit()


class minesweeperGUI:
    def __init__(self, game):
        self.app = QApplication([])
        self.mainWindow = QMainWindow()
        self.centerWidget = QWidget()
        self.mainWindow.setCentralWidget(self.centerWidget)
        self.layoutQVB = QVBoxLayout()
        self.layoutQHB = QHBoxLayout()
        self.gridLayout = QGridLayout()

        self.restartButton = QPushButton("Restart")
        self.restartButton.clicked.connect(lambda GUI=self: restart(self, game))
        self.layoutQHB.addWidget(self.restartButton)
        self.layoutQHB.addLayout(self.layoutQVB)
        self.layoutQVB.addLayout(self.gridLayout)
        #self.centerWidget.setLayout(self.layout)
        self.centerWidget.setLayout(self.layoutQHB)

        self.GUIBoard = []
        for i in range(game.columns):
            self.GUIBoard.append([])
            for j in range(game.rows):
                button = RightClickableButton()
                button.setProperty("column", i)
                button.setProperty("row", game.rows - j - 1)
                button.setStyleSheet("background-color: #DBDBDB")
                button.setFont(QFont('', 20))

                button.clicked.connect(lambda flag=False: self.onClick(flag, game))
                button.rightClicked.connect(lambda flag=True: self.onClick(flag, game))
                button.setFixedSize(QSize(30, 30))
                self.gridLayout.addWidget(button, j, i)

                self.GUIBoard[i].append(button)


    def startGame(self):
        self.mainWindow.show()
        self.app.exec()


    def onClick(self, flag, game):
        if game.gameOver != "Ongoing": return

        clicked = self.mainWindow.sender()
        x = clicked.property("column")
        y = clicked.property("row")
        controller.move(game, x, y, flag)

        for i in range(game.columns):
            for j in range(game.rows):
                if game.revealed[i][j] == 'F':
                    self.GUIBoard[i][game.rows - j - 1].setProperty("text", "F")
                    self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #DBDBDB; color: red")
                elif game.revealed[i][j]:
                    if game.board[i][j] == '0':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF")
                        continue
                    elif game.board[i][j] == '1':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: blue")
                    elif game.board[i][j] == '2':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: green")
                    elif game.board[i][j] == '3':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: red")
                    elif game.board[i][j] == '4':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: purple")
                    elif game.board[i][j] == '5':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: maroon")
                    elif game.board[i][j] == '6':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: turquoise")
                    elif game.board[i][j] == '7':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: black")
                    elif game.board[i][j] == '8':
                        self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #BFBFBF; color: gray")

                    self.GUIBoard[i][game.rows - j - 1].setProperty("text", game.board[i][j])

                else:
                    self.GUIBoard[i][game.rows - j - 1].setProperty("text", "")
                    self.GUIBoard[i][game.rows - j - 1].setStyleSheet("background-color: #DBDBDB")

        if game.gameOver == "Win": self.restartButton.setProperty("text", "Winner!")
        if game.gameOver == "Lose": self.restartButton.setProperty("text", "Loser!")


def restart(GUI, game):
    #No matter what I tried, various .quit() and .exit() functions and even
    #resetting the style for each cell in the board, the "new" game it
    # generates always retains the uncovered cells from the old game
    GUI.mainWindow.close()
    main()



def GUIInit():
    app = QApplication([])
    mainWindow = QMainWindow()
    centerWidget = QWidget()
    mainWindow.setCentralWidget(centerWidget)
    layout = QVBoxLayout()
    centerWidget.setLayout(layout)

    xSpinner = QSpinBox()
    xSpinner.setMaximum(1000)
    xSpinner.setMinimum(1)
    xSpinner.setValue(9)
    ySpinner = QSpinBox()
    ySpinner.setMaximum(1000)
    ySpinner.setMinimum(1)
    ySpinner.setValue(9)
    bSpinner = QSpinBox()
    bSpinner.setMaximum(1000)
    bSpinner.setMinimum(1)
    bSpinner.setValue(10)

    xPrompt = QLabel("Board Width:")
    yPrompt = QLabel("Board Height:")
    bPrompt = QLabel("Number of Bombs:")

    confirmButton = QPushButton("Start")
    confirmButton.clicked.connect(app.quit)

    layout.addWidget(xPrompt)
    layout.addWidget(xSpinner)
    layout.addWidget(yPrompt)
    layout.addWidget(ySpinner)
    layout.addWidget(bPrompt)
    layout.addWidget(bSpinner)
    layout.addWidget(confirmButton)

    mainWindow.show()
    app.exec_()

    x = xSpinner.value()
    y = ySpinner.value()
    b = bSpinner.value()

    return x, y, b


def termInit():
    print("Enter width: ", end="")
    x = int(input())
    print("Enter height: ", end="")
    y = int(input())
    print("Enter number of bombs: ", end="")
    b = int(input())

    return x, y, b


def termGetMove():
    print("Move x: ", end="")
    x = int(input())
    print("Move y: ", end="")
    y = int(input())
    print("Flag? (0 for no, 1 for yes): ", end="")
    flag = bool(int(input()))
    print()

    return x, y, flag


def termPrintGame(game):
    for j in range(game.rows-1, -1, -1):
        for i in range(game.columns): print(game.board[i][j], end=" ")
        print()


def termPrintRevealed(game):
    for j in range(game.rows-1, -1, -1):
        for i in range(game.columns):
            if game.revealed[i][j] and game.revealed[i][j] != 'F': print(game.board[i][j], end=" ")
            elif game.revealed[i][j] == 'F': print('F', end=" ")
            else: print("-", end=" ")
        print()


main()