import random

class minesweeperGame:
    rows = 0
    columns = 0
    bombs = 0
    flags = 0
    numRevealed = 0
    gameOver = "Ongoing"
    board = []
    revealed = []

    def __init__(self, x, y, b):
        # There must be at least 9 empty tiles to ensure
        # a blank opening move
        if b > (x * y - 9):
            print("Invalid number of bombs")
            self.gameOver = "Lose"
            return

        # Fill in basic game info
        self.columns = x
        self.rows = y
        self.bombs = b

        # Generate an empty board and get a list of coordinates
        bombLocations = []
        for i in range(x):
            boardTemp = []
            revealedTemp = []
            for j in range(y):
                boardTemp.append('0')
                revealedTemp.append(False)
                bombLocations.append((i, j))
            self.board.append(boardTemp)
            self.revealed.append(revealedTemp)

        # Randomly places bombs and increases the number on adjacent tiles
        for i in range(b):
            nextBomb = bombLocations.pop(random.randrange(0, len(bombLocations)))
            self.board[nextBomb[0]][nextBomb[1]] = 'B'

        self.determineNumbers()


    def isValid(self, x, y, flag):
        if x >= self.columns or y >= self.rows or (self.revealed[x][y] == 'F' and not flag): return False
        return True


    def flag(self, x, y):
        if not self.revealed[x][y]: self.revealed[x][y] = 'F'
        else: self.revealed[x][y] = False


    def revealAdj(self, x, y):
        flagCount = 0
        revealAdj = []
        for adj in self.getAdjacent(x, y):
            if self.revealed[adj[0]][adj[1]] == 'F':
                flagCount += 1
            else:
                revealAdj.append(adj)
        if flagCount == int(self.board[x][y]):
            for adj in revealAdj:
                if self.board[adj[0]][adj[1]] == 'B':
                    self.gameOver = "Lose"
                    return
                self.reveal(adj[0], adj[1])


    def blankStart(self, x, y):
        toBeCleared = self.getAdjacent(x, y)
        toBeCleared.append((x, y))
        clearSet = set(toBeCleared)
        fourNonBombs = []

        # Gets all viable positions to move bombs
        for i in range(self.columns):
            for j in range(self.rows):
                if self.board[i][j] != 'B' and (i, j) not in clearSet:
                    fourNonBombs.append((i, j))

        # Swap the position of the bombs adjacent to the
        # starting position to a random position
        for location in toBeCleared:
            if self.board[location[0]][location[1]] != 'B': continue
            self.board[location[0]][location[1]] = '0'
            nextBomb = fourNonBombs.pop(random.randrange(len(fourNonBombs)))
            self.board[nextBomb[0]][nextBomb[1]] = 'B'

        # Re-number the board
        self.determineNumbers()


    def isWin(self, x, y):
        if self.numRevealed == self.rows * self.columns - self.bombs:
            self.gameOver = "Win"
        if self.board[x][y] == 'B':
            self.gameOver = "Lose"


    def reveal(self, x, y):
        if self.revealed[x][y]: return

        self.revealed[x][y] = True
        self.numRevealed += 1
        if self.board[x][y] != '0': return

        for adj in self.getAdjacent(x, y):
            if not self.revealed[adj[0]][adj[1]]:
                self.reveal(adj[0], adj[1])


    def increment(self, x, y, amount=1):
        if self.board[x][y] != 'B':
            self.board[x][y] = chr(ord(self.board[x][y]) + amount)


    # Returns a set of tuples of the (x, y) coordinates of
    # valid adjacent squares
    def getAdjacent(self, x, y, diagonals=True):
        output = []
        if x+1 < self.columns:
            output.append((x+1, y))
        if x-1 >= 0:
            output.append((x-1, y))
        if y+1 < self.rows:
            output.append((x, y+1))
        if y-1 >= 0:
            output.append((x, y-1))

        if not diagonals: return output

        if x+1 < self.columns and y+1 < self.rows:
            output.append((x+1, y+1))
        if x+1 < self.columns and y-1 >= 0:
            output.append((x+1, y-1))
        if x-1 >= 0 and y+1 < self.rows:
            output.append((x-1, y+1))
        if x-1 >= 0 and y-1 >= 0:
            output.append((x-1, y-1))

        return output


    def determineNumbers(self):
        for i in range(self.columns):
            for j in range(self.rows):
                if self.board[i][j] == 'B': continue
                self.board[i][j] = '0'
                for adj in self.getAdjacent(i, j):
                    if self.board[adj[0]][adj[1]] == 'B':
                        self.increment(i, j)


    def move(self, x, y, flag=False):
        if not self.isValid(x, y, flag):
            print("Invalid Move")
            return

        if flag:
            self.flag(x, y)
            return

        if self.revealed[x][y] and self.revealed != 'F': self.revealAdj(x, y)
        if self.numRevealed == 0 and self.board[x][y] != '0': self.blankStart(x, y)
        self.reveal(x, y)
        self.isWin(x, y)